<?php

declare(strict_types=1);

namespace app\controllers;

use app\models\AccountEdgeNotification;
use yii;
use yii\filters\AccessControl;
use yii\web\Controller;
use yii\web\Response;

use app\models\Supply;
use app\models\Order;
use app\models\AccountEdgeOrder;
use app\models\OrderEdgeSupply;
use Exception;

class OrderController extends Controller
{
    public function behaviors()
    {
        return [
            'access' => [
                'class' => AccessControl::class,
                'only' => ['index'],
                'rules' => [
                    [
                        'allow' => true,
                        'roles' => ['@']
                    ]
                ]
            ]
        ];
    }

    public function actionIndex()
    {
        // Инициализация
        $orders = Order::search(type: 'all', limit: 10, page: 1, select: '{account_edge_order, order}');

        if (yii::$app->request->isPost) {
            // POST-запрос

            yii::$app->response->format = Response::FORMAT_JSON;

            return [
                'main' => $this->renderPartial('/orders/index', compact('orders')),
                'title' => 'Заказы',
                'redirect' => '/orders',
                '_csrf' => yii::$app->request->getCsrfToken()
            ];
        }

        return $this->render('/orders/index', compact('orders'));
    }

    /**
     * Запись
     */
    public function actionWrite(): string|array|null
    {
        if (yii::$app->request->isPost) {
            // POST-запрос

            // Инициализация входных данных
            $account = yii::$app->user;
            $supplies = yii::$app->request->post('supplies');

            yii::$app->response->format = Response::FORMAT_JSON;

            // Инициализация возврата по умолчанию
            $return = [
                '_csrf' => yii::$app->request->getCsrfToken()
            ];

            if (is_null($supplies)) {
                // 501 Not Implemented
                yii::$app->response->statusCode = 501;
            }

            if (!is_null($supplies)) {
                // Переданы поставки для записи

                if (!$model = Order::search($account)) {
                    // Корзина не найдена (текущий заказ)

                    // Инициализация
                    $model = new Order();
                    $model->save() or throw new Exception('Не удалось инициализировать заказ');

                    // Запись ребра: АККАУНТ -> ЗАКАЗ
                    AccountEdgeOrder::write($account->id, $model->readId(), 'current') or $model->addError('errors', 'Не удалось инициализировать ребро: АККАУНТ -> ЗАКАЗ');
                }

                // Проверка входных данных
                if (!is_array($supplies)) {
                    // Неверные входные данные

                    // Запись ошибки
                    $model->addError('errors', 'Переменная должна быть массивом');
                }

                // Если запись не удалась, то вернуть код: 500 Internal Server Error
                $model->write($supplies) or yii::$app->response->statusCode = 500;
            }

            return $return;
        }

        yii::$app->response->statusCode = 500;

        return null;
    }

    /**
     * Удаление
     */
    public function actionDelete(): string|array|null
    {
        // Инициализация
        $targets = yii::$app->request->post('targets') ?? yii::$app->request->get('targets');
        $page = yii::$app->request->get('page') ?? yii::$app->request->post('page') ?? 1;
        $account = yii::$app->user;

        if ($targets) {
            // Удаление выбранных целей (удаление из заказа)

            foreach ($targets as $target) {

                // Инициализация
                $model = Order::search();

                // Удаление
                $model->deleteEdge($target);
            }
        } else {
            // Целью подразумевается сам заказ (удаление заказа)

            // Инициализация корзины (текущего заказа)
            $model = Order::search();

            // Поиск
            $edge = AccountEdgeOrder::searchByVertex($account->id, $model->readId(), 'current');

            // Запись
            $edge->type = 'reserved';

            // Отправка
            $edge->update();
        }

        // Инициализация содержимого корзины
        $supplies = $model->content(10, $page);

        if (yii::$app->request->isPost) {
            // POST-запрос

            yii::$app->response->format = Response::FORMAT_JSON;

            return [
                'main' => $this->renderPartial('/cart/index', compact('model', 'supplies')),
                'title' => 'Корзина',
                'redirect' => '/cart',
                '_csrf' => yii::$app->request->getCsrfToken()
            ];
        }

        return $this->render('/cart/index', compact('model', 'supplies'));
    }

    /**
     * Оплата
     */
    public function actionPay(): string|array|null
    {
        // Инициализация
        $model = Order::search();

        // Поиск ребра
        $edge = AccountEdgeOrder::searchByVertex(yii::$app->user->id, $model->readId(), 'current');

        // Запись
        $edge->type = 'accepted';

        // Отправка изменений
        $edge->update();

        // Инициализация
        $orders = Order::search(type: 'all', limit: 10, page: 1, select: '{account_edge_order, order}');

        if (yii::$app->request->isPost) {
            // POST-запрос

            yii::$app->response->format = Response::FORMAT_JSON;

            return [
                'main' => $this->renderPartial('/orders/index', compact('orders')),
                'title' => 'Заказы',
                'redirect' => '/orders',
                '_csrf' => yii::$app->request->getCsrfToken()
            ];
        }

        return $this->render('/orders/index', compact('orders'));
    }
}
